#ifndef IMUTILS_INCLUDE_IMUTILS_METADBCONNECTORFACTORY_H_
#define IMUTILS_INCLUDE_IMUTILS_METADBCONNECTORFACTORY_H_

#include <imutils/MetaDBConnector.h>
#include <memory>

namespace erad::db {

/**
 * Factory class to create instances of MetaDBConnector.
 *
 * Allows dynamic creation of connector implementations based on type.
 */
class MetaDBConnectorFactory
{
	public:

		/**
		 * Factory method to create a connector instance based on the specified repository type.
		 *
		 * @param repositoryType enum indicating the type of repository to connect to.
		 *
		 * @return unique pointer to a MetaDBConnector instance appropriate for the repository type.
		 */
		static std::unique_ptr<MetaDBConnector> createConnector( erad::db::RepositoryType repositoryType );
};

}

#endif
