#ifndef CLOUDACCESS_H
#define CLOUDACCESS_H

#include <string>

#include "utils/CloudDicomStore.h"

/**
 * This is the class used to get an access to the cloud storage.
 * It create and returns cloud store object
 */
class CloudAccess {
		/*
		 * To keep Project name details
		 */
		std::string m_HealthcareProject;
		/*
		 * To keep region name details
		 */
		std::string m_HealthcareRegion;
		/*
		 * To keep Dataset name details
		 */
		std::string m_HealthcareDataset;
		/*
		 * To keep dicom store name details
		 */
		std::string m_HealthcareDicomId;
		/*
		 * Tocken details
		 */
		std::string m_Token;

	public:

		/**
		 * Constructor for CloudAccess
		 * @param studyUID - String buffer to read from
		 * @return nil
		 * @exception - std::runtime_error will be thrown when token generation failed 
		 * 				or default DICOM store configuration is not found.
		 */
		CloudAccess();
		
		/**
		 * Destructor for CloudAccess
		 */
		virtual ~CloudAccess();

		/**
		 * Function to get the cloud dicom store path for a study
		 */
		std::string getCloudDicomStorePath( const std::string& studyUID );

		/**
		 * Function to get the Dicom store object for a study
		 */
		std::unique_ptr<CloudDicomStore> getDicomStoreForStudy( const std::string& studyUID );

		/**
		 * Function to get the DICOM store object for a specific device
		 * @param deviceId - Device Id
		 * @return A pointer to the CloudDicomStore instance representing the DICOM store for the device.
		 * @throws IMException
		 */
		std::unique_ptr<CloudDicomStore> getDicomStoreForDevice( const std::string& deviceId );
		
		/**
		 * Function to get the Dicom store object from dicom store path
		 */
		std::unique_ptr<CloudDicomStore> getDicomStoreFromPath( const std::string& dicomStorePath );

		/**
		 * Function to get the cloud DICOM store path for a specific device
		 * @param deviceId - Device Id
		 * @return Path to the DICOM store for this device
		 */
		std::string getDeviceDicomStorePath( const std::string& deviceId );
};

#endif // CLOUDACCESS_H
