/*
 * imbase64.h
 *
 *  Created on: Dec 6, 2024
 *      Author: medsrv
 */

#ifndef UTILS_INCLUDE_UTILS_IMBASE64_H_
#define UTILS_INCLUDE_UTILS_IMBASE64_H_

#include <string>

namespace base64 {

class Base64 {

	private:

		static const std::string base64_chars;/* =
		 "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
		 "abcdefghijklmnopqrstuvwxyz"
		 "0123456789+/";*/

		static const std::vector<char> base64_codes; /* = { 0x00, 0x00, 0x00, 0x00,
		 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3E, 0x00, 0x00, 0x00, 0x3F,
		 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x00,
		 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x02, 0x03, 0x04,
		 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F,
		 0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x00,
		 0x00, 0x00, 0x00, 0x00, 0x00, 0x1A, 0x1B, 0x1C, 0x1D, 0x1E, 0x1F,
		 0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2A,
		 0x2B, 0x2C, 0x2D, 0x2E, 0x2F, 0x30, 0x31, 0x32, 0x33, 0x00, 0x00,
		 0x00, 0x00, 0x00 };*/

	public:
		static std::string encode( unsigned char const* pBuffer, size_t nLength ) {

			std::string sEncoded;
			sEncoded.resize( ( ( nLength + 2 ) / 3 ) * 4 );
			char* pData = &sEncoded[ 0 ];
			unsigned int nId = 0;

			while ( nLength >= 3 ) {
				unsigned int nD = ( pBuffer[ 0 ] << 16 ) | ( pBuffer[ 1 ] << 8 )
						| ( pBuffer[ 2 ] );

				pData[ nId + 3 ] = base64_chars[ ( nD & 0x3f ) ];
				pData[ nId + 2 ] = base64_chars[ ( ( nD >> 6 ) & 0x3f ) ];
				pData[ nId + 1 ] = base64_chars[ ( ( nD >> 12 ) & 0x3f ) ];
				pData[ nId + 0 ] = base64_chars[ ( ( nD >> 18 ) & 0x3f ) ];

				pBuffer += 3;
				nId += 4;
				nLength -= 3;
			}

			if ( nLength > 0 ) {

				unsigned int nD = ( pBuffer[ 0 ] << 16 );
				if ( nLength > 1 ) {
					nD |= ( pBuffer[ 1 ] << 8 );
					if ( nLength > 2 ) {
						nD |= ( pBuffer[ 2 ] );
					}
				}

				if ( nLength > 2 ) {
					pData[ nId + 3 ] = base64_chars[ ( nD & 0x3f ) ];
				}
				else {
					pData[ nId + 3 ] = '=';
				}
				if ( nLength > 1 ) {
					pData[ nId + 2 ] = base64_chars[ ( ( nD >> 6 ) & 0x3f ) ];
				}
				else {
					pData[ nId + 2 ] = '=';
				}
				pData[ nId + 1 ] = base64_chars[ ( ( nD >> 12 ) & 0x3f ) ];
				pData[ nId + 0 ] = base64_chars[ ( ( nD >> 18 ) & 0x3f ) ];
			}

			return sEncoded;
		}

		static size_t decode_len( const std::string& sEncoded ) {
			const unsigned int sInputLen = sEncoded.length();
			unsigned int nLen = ( sInputLen / 4 ) * 3;

			if ( nLen > 0 ) {
				if ( sEncoded[ sInputLen - 1 ] == '=' ) {
					nLen--;
					if ( sEncoded[ sInputLen - 2 ] == '=' ) {
						nLen--;
					}
				}
			}
			return nLen;
		}

		static bool decode( const std::string& sEncoded, unsigned char* pBuffer ) {
			/*
			 // table code generator
			 for (unsigned int idx = 0; idx < 128; idx++) {
			 size_t fid = base64_chars.find( idx);
			 if (std::string::npos == fid) {
			 sEncoded += " 0x00,";
			 } else {
			 char aaa[7];
			 sprintf( aaa, " 0x%02X,", fid);
			 sEncoded += aaa;
			 }
			 }
			 */

			// doesn't handle the error, the base64_codes can be overindexed.
			unsigned int nId = 0;
			const unsigned int nLen = sEncoded.length();

			while ( nId < nLen - 4 ) {
				unsigned int nD = base64_codes[ sEncoded[ nId + 3 ] ]
						| ( base64_codes[ sEncoded[ nId + 2 ] ] << 6 )
						| ( base64_codes[ sEncoded[ nId + 1 ] ] << 12 )
						| ( base64_codes[ sEncoded[ nId + 0 ] ] << 18 );

				pBuffer[ 2 ] = nD & 0xff;
				pBuffer[ 1 ] = ( nD >> 8 ) & 0xff;
				pBuffer[ 0 ] = ( nD >> 16 ) & 0xff;

				nId += 4;
				pBuffer += 3;
			}

			if ( nId < nLen ) {
				unsigned int nD = base64_codes[ sEncoded[ nId + 3 ] ]
						| ( base64_codes[ sEncoded[ nId + 2 ] ] << 6 )
						| ( base64_codes[ sEncoded[ nId + 1 ] ] << 12 )
						| ( base64_codes[ sEncoded[ nId + 0 ] ] << 18 );

				pBuffer[ 0 ] = ( nD >> 16 ) & 0xff;
				if ( sEncoded[ nId + 2 ] != '=' ) {
					pBuffer[ 1 ] = ( nD >> 8 ) & 0xff;
					if ( sEncoded[ nId + 3 ] != '=' ) {
						pBuffer[ 2 ] = nD & 0xff;
					}
				}
			}

			return true;
		}

};

const std::string Base64::base64_chars =
		"ABCDEFGHIJKLMNOPQRSTUVWXYZ"
				"abcdefghijklmnopqrstuvwxyz"
				"0123456789+/";

const std::vector<char> Base64::base64_codes = { 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3E, 0x00, 0x00, 0x00, 0x3F,
		0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x02, 0x03, 0x04,
		0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F,
		0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x1A, 0x1B, 0x1C, 0x1D, 0x1E, 0x1F,
		0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2A,
		0x2B, 0x2C, 0x2D, 0x2E, 0x2F, 0x30, 0x31, 0x32, 0x33, 0x00, 0x00,
		0x00, 0x00, 0x00 };

}

#endif /* UTILS_INCLUDE_UTILS_IMBASE64_H_ */
